@@ -12,16 +12,18 @@ Please refer to https://github.com/QSD-Group/EXPOsan/blob/main/LICENSE.txt
 for license details.
 '''
 
-from . import _cmps, systems, models, analyses
+from . import _cmps, _lca_data, systems, models, analyses
 
 from ._cmps import *
+from ._lca_data import *
 from .systems import *
 from .models import *
 from .analyses import *
 
 __all__ = (
 	*_cmps.__all__,
+    *_lca_data.__all__,
 	*systems.__all__,
     *models.__all__,
     *analyses.__all__,
-	)
+	)
\ No newline at end of file
