@@ -12,11 +12,11 @@ Please refer to https://github.com/QSD-Group/EXPOsan/blob/main/LICENSE.txt
 for license details.
 
 TODOs:
-    [1] Recheck COD calculation acorss units once WasteStream is ready
+    [1] Recheck COD calculation acorss units once WasteStream is ready.
 
 Questions:
     [1] WWTP power consumption very low, only 6.5/0.8 kWh per hour for
-        existing/alternative WWTP
+        existing/alternative WWTP.
 
 '''
 
@@ -30,6 +30,8 @@ from sklearn.linear_model import LinearRegression as LR
 from qsdsan import sanunits as su
 from qsdsan import WasteStream, ImpactIndicator, ImpactItem, StreamImpactItem, SimpleTEA, LCA
 from exposan.bwaise._cmps import cmps
+from exposan.bwaise._lca_data import load_lca_data
+
 
 # =============================================================================
 # Unit parameters
@@ -40,19 +42,6 @@ qs.CEPCI = qs.CEPCI_by_year[2018]
 qs.set_thermo(cmps)
 bst.speed_up()
 
-items = ImpactItem._items
-
-if not items: # prevent from reloading
-    import os
-    path = os.path.dirname(os.path.realpath(__file__)) + '/data'
-    ImpactIndicator.load_indicators_from_file(path+'/impact_indicators.tsv')
-    item_path = path+'/impact_items.xlsx'
-    ImpactItem.load_items_from_excel(item_path)
-    del os
-
-GWP = qs.ImpactIndicator._indicators['GWP']
-
-
 household_size = 4
 get_household_size = lambda: household_size
 household_per_toilet = 4
@@ -147,6 +136,13 @@ GWP_dct = {
     'Biogas': -3*get_biogas_factor()
     }
 
+items = ImpactItem.get_all_items()
+
+if not items.get('Excavation'): # prevent from reloading
+    load_lca_data('original')
+
+GWP = ImpactIndicator.get_indicator('GWP')
+
 bst.PowerUtility.price = price_dct['Electricity']
 items['Concrete'].price = price_dct['Concrete']
 items['Steel'].price = price_dct['Steel']
@@ -155,46 +151,68 @@ items['Steel'].price = price_dct['Steel']
 # Universal units and functions
 # =============================================================================
 
-CH4_item = StreamImpactItem(ID='CH4_item', GWP=GWP_dct['CH4'])
-N2O_item = StreamImpactItem(ID='N2O_item', GWP=GWP_dct['N2O'])
-N_item = StreamImpactItem(ID='N_item', GWP=GWP_dct['N'])
-P_item = StreamImpactItem(ID='P_item', GWP=GWP_dct['P'])
-K_item = StreamImpactItem(ID='K_item', GWP=GWP_dct['K'])
-biogas_item = StreamImpactItem(ID='biogas_item', GWP=GWP_dct['Biogas'])
-e_item = ImpactItem(ID='e_item', functional_unit='kWh', GWP=GWP_dct['Electricity'])
+def batch_create_stream_items(kind):
+    if kind == 'original':
+        for k, v in GWP_dct.items():
+            if k == 'Electricity':
+                ImpactItem(ID='E_item', functional_unit='kWh', GWP=v)
+            else:
+                StreamImpactItem(ID=f'{k}_item', GWP=v)
+    elif kind == 'new':
+        E_factor = {'GW2ECO': 0.000000000532, # global warming to ecosystem
+                    'GW2HH': 0.0000000812, # global warming to human health
+                    'OD2HH': 0.00134} # stratospheric ozone depletion to human health
+        H_factor = {'GW2ECO': 0.0000000028, 'GW2HH': 0.000000928, 'OD2HH': 0.000531}
+        I_factor = {'GW2ECO': 0.000000025, 'GW2HH': 0.0000125, 'OD2HH': 0.000237}
+
+        StreamImpactItem(ID=f'CH4_item',
+                         E_ClimateChangeEcosystems=E_factor['GW2ECO']*4.8,
+                         E_ClimateChangeHumanHealth=E_factor['GW2HH']*4.8,
+                         H_ClimateChangeEcosystems=H_factor['GW2ECO']*34,
+                         H_ClimateChangeHumanHealth=H_factor['GW2HH']*34,
+                         I_ClimateChangeEcosystems=I_factor['GW2ECO']*84,
+                         I_ClimateChangeHumanHealth=I_factor['GW2HH']*84
+                         )
+        StreamImpactItem(ID=f'N2O_item',
+                         E_ClimateChangeEcosystems=E_factor['GW2ECO']*78.8,
+                         E_ClimateChangeHumanHealth=E_factor['GW2HH']*78.8,
+                         H_ClimateChangeEcosystems=H_factor['GW2ECO']*298,
+                         H_ClimateChangeHumanHealth=H_factor['GW2HH']*298,
+                         I_ClimateChangeEcosystems=I_factor['GW2ECO']*264,
+                         I_ClimateChangeHumanHealth=I_factor['GW2HH']*264,
+                         E_OzoneDepletion=E_factor['OD2HH']*0.017,
+                         H_OzoneDepletion=H_factor['OD2HH']*0.011,
+                         I_OzoneDepletion=I_factor['OD2HH']*0.007
+                         )
+    else:
+        raise ValueError(f'`kind` can only be "original" or "new", not "{kind}".')
+
+batch_create_stream_items(kind='original')
 
 def batch_create_streams(prefix):
     stream_dct = {}
     stream_dct['CH4'] = WasteStream(f'{prefix}_CH4', phase='g',
-                                    impact_item=CH4_item.copy(set_as_source=True,
-                                                              register=False))
+                                    stream_impact_item=items['CH4_item'].copy(set_as_source=True))
     stream_dct['N2O'] = WasteStream(f'{prefix}_N2O', phase='g',
-                                    impact_item=N2O_item.copy(set_as_source=True,
-                                                              register=False))
+                                    stream_impact_item=items['N2O_item'].copy(set_as_source=True))
     stream_dct['liq_N'] = WasteStream(f'{prefix}_liq_N', phase='l', price=price_dct['N'],
-                                      impact_item=N_item.copy(set_as_source=True,
-                                                              register=False))
+                                      stream_impact_item=items['N_item'].copy(set_as_source=True))
     stream_dct['sol_N'] = WasteStream(f'{prefix}_sol_N', phase='l', price=price_dct['N'],
-                                      impact_item=N_item.copy(set_as_source=True,
-                                                              register=False))
+                                      stream_impact_item=items['N_item'].copy(set_as_source=True))
     stream_dct['liq_P'] = WasteStream(f'{prefix}_liq_P', phase='l', price=price_dct['P'],
-                                      impact_item=P_item.copy(set_as_source=True,
-                                                              register=False))
+                                      stream_impact_item=items['P_item'].copy(set_as_source=True))
     stream_dct['sol_P'] = WasteStream(f'{prefix}_sol_P', phase='l', price=price_dct['P'],
-                                      impact_item=P_item.copy(set_as_source=True,
-                                                              register=False))
+                                      stream_impact_item=items['P_item'].copy(set_as_source=True))
     stream_dct['liq_K'] = WasteStream(f'{prefix}_liq_K', phase='l', price=price_dct['K'],
-                                      impact_item=K_item.copy(set_as_source=True,
-                                                              register=False))
+                                      stream_impact_item=items['K_item'].copy(set_as_source=True))
     stream_dct['sol_K'] = WasteStream(f'{prefix}_sol_K', phase='l', price=price_dct['K'],
-                                      impact_item=K_item.copy(set_as_source=True,
-                                                              register=False))
+                                      stream_impact_item=items['K_item'].copy(set_as_source=True))
     return stream_dct
 
-def add_fugitive_items(unit, item):
+def add_fugitive_items(unit, item_ID):
     unit._run()
     for i in unit.ins:
-        i.impact_item = item.copy(set_as_source=True, register=False)
+        i.stream_impact_item = items[item_ID].copy(set_as_source=True)
 
 # Costs of WWTP units have been considered in the lumped unit
 def clear_unit_costs(sys):
@@ -219,7 +237,6 @@ def adjust_NH3_loss(unit):
 
 flowsheetA = bst.Flowsheet('sysA')
 bst.main_flowsheet.set_flowsheet(flowsheetA)
-# breakpoint()
 streamsA = batch_create_streams('A')
 
 #################### Human Inputs ####################
@@ -293,11 +310,11 @@ A9 = su.CropApplication('A9', ins=A7-0, outs=('liquid_fertilizer', 'reuse_loss')
 A9.specification = lambda: adjust_NH3_loss(A9)
 
 A10 = su.Mixer('A10', ins=(A2-2, A5-2, A6-1, A7-1, A8-2), outs=streamsA['CH4'])
-A10.specification = lambda: add_fugitive_items(A10, CH4_item)
-A10.line = 'fugitive CH4 mixer' 
-        
+A10.specification = lambda: add_fugitive_items(A10, 'CH4_item')
+A10.line = 'fugitive CH4 mixer'
+
 A11 = su.Mixer('A11', ins=(A2-3, A5-3, A6-2, A7-2, A8-3), outs=streamsA['N2O'])
-A11.specification = lambda: add_fugitive_items(A11, N2O_item)
+A11.specification = lambda: add_fugitive_items(A11, 'N2O_item')
 A11.line = 'fugitive N2O mixer'
 
 A12 = su.ComponentSplitter('A12', ins=A8-0,
@@ -312,7 +329,7 @@ A13 = su.ComponentSplitter('A13', ins=A9-0,
 
 ############### Simulation, TEA, and LCA ###############
 sysA = bst.System('sysA', path=(A1, A2, A3, treatA, A9, A10, A11, A12, A13))
-# breakpoint()
+
 teaA = SimpleTEA(system=sysA, discount_rate=get_discount_rate(), start_year=2018,
                  lifetime=get_A4_lifetime(), uptime_ratio=1, lang_factor=None,
                  annual_maintenance=0, annual_labor=12*3e6*12/get_exchange_rate(),
@@ -320,7 +337,7 @@ teaA = SimpleTEA(system=sysA, discount_rate=get_discount_rate(), start_year=2018
 
 lcaA = LCA(system=sysA, lifetime=get_A4_lifetime(), lifetime_unit='yr', uptime_ratio=1,
             # Assuming all additional WWTP OPEX from electricity
-            e_item=lambda: A4.power_utility.rate*(365*24)*8)
+            E_item=lambda: A4.power_utility.rate*(365*24)*8)
 
 
 # %%
@@ -333,8 +350,7 @@ flowsheetB = bst.Flowsheet('sysB')
 bst.main_flowsheet.set_flowsheet(flowsheetB)
 streamsB = batch_create_streams('B')
 streamsB['biogas'] = WasteStream('B_biogas', phase='g', price=price_dct['Biogas'],
-                                 impact_item=biogas_item.copy(set_as_source=True,
-                                                              register=False))
+                                 stream_impact_item=items['Biogas_item'].copy(set_as_source=True))
 
 #################### Human Inputs ####################
 B1 = su.Excretion('B1', outs=('urine', 'feces'))
@@ -400,11 +416,11 @@ B9 = su.CropApplication('B9', ins=B7-0, outs=('liquid_fertilizer', 'reuse_loss')
 B9.specification = lambda: adjust_NH3_loss(B9)
 
 B10 = su.Mixer('B10', ins=(B2-2, B5-2, B7-1, B8-2), outs=streamsB['CH4'])
-B10.specification = lambda: add_fugitive_items(B10, CH4_item)
+B10.specification = lambda: add_fugitive_items(B10, 'CH4_item')
 B10.line = 'fugitive CH4 mixer'
 
 B11 = su.Mixer('B11', ins=(B2-3, B5-3, B7-2, B8-3), outs=streamsB['N2O'])
-B11.specification = lambda: add_fugitive_items(B11, N2O_item)
+B11.specification = lambda: add_fugitive_items(B11, 'N2O_item')
 B11.line = 'fugitive N2O mixer'
 
 B12 = su.ComponentSplitter('B12', ins=B8-0,
@@ -439,7 +455,7 @@ teaB = SimpleTEA(system=sysB, discount_rate=get_discount_rate(), start_year=2018
 
 lcaB = LCA(system=sysB, lifetime=get_B4_lifetime(), lifetime_unit='yr', uptime_ratio=1,
            # Assuming all additional WWTP OPEX from electricity
-           e_item=lambda: B4.power_utility.rate*(365*24)*10)
+           E_item=lambda: B4.power_utility.rate*(365*24)*10)
 
 
 # %%
@@ -534,11 +550,11 @@ C9 = su.CropApplication('C9', ins=C7-0, outs=('liquid_fertilizer', 'reuse_loss')
 C9.specification = lambda: adjust_NH3_loss(C9)
 
 C10 = su.Mixer('C10', ins=(C2-4, C6-1, C7-1, C8-2), outs=streamsC['CH4'])
-C10.specification = lambda: add_fugitive_items(C10, CH4_item)
+C10.specification = lambda: add_fugitive_items(C10, 'CH4_item')
 C10.line = 'fugitive CH4 mixer'
 
 C11 = su.Mixer('C11', ins=(C2-5, C6-2, C7-2, C8-3), outs=streamsC['N2O'])
-C11.specification = lambda: add_fugitive_items(C11, N2O_item)
+C11.specification = lambda: add_fugitive_items(C11, 'N2O_item')
 C11.line = 'fugitive N2O mixer'
 
 C12 = su.ComponentSplitter('C12', ins=C8-0,
@@ -561,15 +577,35 @@ teaC = SimpleTEA(system=sysC, discount_rate=get_discount_rate(), start_year=2018
 
 lcaC = LCA(system=sysC, lifetime=get_C5_lifetime(), lifetime_unit='yr', uptime_ratio=1,
            # Assuming all additional WWTP OPEX from electricity
-           e_item=lambda: C5.power_utility.rate*(365*24)*8)
+           E_item=lambda: C5.power_utility.rate*(365*24)*8)
 
 
 # %%
 
 # =============================================================================
-# Summarizing Functions
+# Util functions
 # =============================================================================
 
+def update_lca_data(kind):
+    '''
+    Load impact indicator and impact item data.
+
+    Parameters
+    ----------
+    kind : str
+        "original" loads the data from Trimmer et al.
+        (TRACI, ecoinvent v3.2),
+        "new" loads the data for ReCiPe and TRACI
+        (ecoinvent 3.7.1, at the point of substitution).
+    '''
+
+    load_lca_data(kind)
+    batch_create_stream_items(kind)
+
+    for v in items['Biogas_item'].CFs.values():
+        v *= get_biogas_factor()
+
+
 def get_total_inputs(unit):
     if len(unit.ins) == 0: # Excretion units do not have ins
         ins = unit.outs
@@ -610,8 +646,8 @@ def get_stream_emissions(streams=None, hr=365*24, ppl=1):
     emission = {}
     factor = hr / ppl
     for i in streams:
-        if not i.impact_item: continue
-        emission[f'{i.ID}'] = i.F_mass*i.impact_item.CFs['GlobalWarming']*factor
+        if not i.stream_impact_item: continue
+        emission[f'{i.ID}'] = i.F_mass*i.stream_impact_item.CFs['GlobalWarming']*factor
     return emission
 
 sys_dct = {
@@ -629,12 +665,13 @@ sys_dct = {
 def cache_recoveries(sys):
     total_COD = get_total_inputs(sys_dct['input_unit'][sys.ID])['COD']
     ppl = sys_dct['ppl'][sys.ID]
+
     if sys_dct['gas_unit'][sys.ID]:
         gas_mol = sys_dct['gas_unit'][sys.ID].outs[0].imol['CH4']
         gas_COD = gas_mol*1e3*get_biogas_energy()*365*24/14e3/ppl/total_COD
-        # breakpoint()
     else:
         gas_COD = 0
+
     cache = {
         'liq': get_recovery(ins=sys_dct['input_unit'][sys.ID],
                             outs=sys_dct['liq_unit'][sys.ID].ins,
@@ -667,7 +704,7 @@ def get_summarizing_fuctions():
     func_dct['get_constr_GWP'] = \
         lambda lca, ppl: lca.total_construction_impacts[ind]/lca.lifetime/ppl
     func_dct['get_trans_GWP'] = \
-        lambda lca, ppl: lca.total_transportation_impacts[ind]/lca.lifetime/ppl  
+        lambda lca, ppl: lca.total_transportation_impacts[ind]/lca.lifetime/ppl
     func_dct['get_direct_emission_GWP'] = \
         lambda lca, ppl: lca.get_stream_impacts(stream_items=lca.stream_inventory, kind='direct_emission')[ind] \
             /lca.lifetime/ppl
@@ -704,12 +741,12 @@ def print_summaries(systems):
         print('\n')
         lca = sys_dct['LCA'][sys.ID]
         lca.show()
-        
+
         unit = f'{currency}/cap/yr'
         print(f'\nNet cost: {func["get_annual_cost"](tea, ppl):.1f} {unit}.')
         print(f'Capital: {func["get_annual_CAPEX"](tea, ppl):.1f} {unit}.')
         print(f'Operating: {func["get_annual_OPEX"](tea, ppl):.1f} {unit}.')
-        
+
         unit = f'{GWP.unit}/cap/yr'
         print(f'\nNet emission: {func["get_annual_GWP"](lca, ppl):.1f} {unit}.')
         print(f'Construction: {func["get_constr_GWP"](lca, ppl):.1f} {unit}.')
@@ -739,15 +776,8 @@ def save_all_reports():
             i.save_report(f'{path}/{i.system.ID}_lca.xlsx')
 
 __all__ = ('sysA', 'sysB', 'sysC', 'teaA', 'teaB', 'teaC', 'lcaA', 'lcaB', 'lcaC',
-           'print_summaries', 'save_all_reports',
+           'print_summaries', 'save_all_reports', 'update_lca_data',
            *(i.ID for i in sysA.units),
            *(i.ID for i in sysB.units),
            *(i.ID for i in sysC.units),
-           )
-
-
-
-
-
-
-
+           )
\ No newline at end of file
